@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ const char *optreal_value(int p);
 const char *optalias_value(int p);
 const char *opttype_value(int p);
 const char *opthelp_value(int p);
-char *hts_gethome(void);
+const char *hts_gethome(void);
 void expand_home(String * str);
 #endif
 
