@@ -1,6 +1,6 @@
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,14 +38,15 @@ Please visit our Website: http://www.httrack.com
 /* specific definitions */
 #include "htsglobal.h"
 #include "htslib.h"
+#include "htscore.h"
 
 /* END specific definitions */
 
 // gestion des cookie
 // ajoute, dans l'ordre
 // !=0 : erreur
-int cookie_add(t_cookie * cookie, char *cook_name, char *cook_value,
-               char *domain, char *path) {
+int cookie_add(t_cookie * cookie, const char *cook_name, const char *cook_value,
+               const char *domain, const char *path) {
   char buffer[8192];
   char *a = cookie->data;
   char *insert;
@@ -53,20 +54,20 @@ int cookie_add(t_cookie * cookie, char *cook_name, char *cook_value,
 
   // effacer éventuel cookie en double
   cookie_del(cookie, cook_name, domain, path);
-  if ((int) strlen(cook_value) > 1024)
+  if (strlen(cook_value) > 1024)
     return -1;                  // trop long
-  if ((int) strlen(cook_name) > 256)
+  if (strlen(cook_name) > 256)
     return -1;                  // trop long
-  if ((int) strlen(domain) > 256)
+  if (strlen(domain) > 256)
     return -1;                  // trop long
-  if ((int) strlen(path) > 256)
+  if (strlen(path) > 256)
     return -1;                  // trop long
-  if ((int) (strlen(cookie->data)
+  if (strlen(cookie->data)
              + strlen(cook_value)
              + strlen(cook_name)
              + strlen(domain)
              + strlen(path)
-             + 256) > cookie->max_len)
+             + 256 > cookie->max_len)
     return -1;                  // impossible d'ajouter
 
   insert = a;                   // insérer ici
@@ -99,7 +100,7 @@ int cookie_add(t_cookie * cookie, char *cook_name, char *cook_value,
   strcatbuff(cook, "\t");
   strcatbuff(cook, cook_value);
   strcatbuff(cook, "\n");
-  if (!(((int) strlen(cookie->data) + (int) strlen(cook)) < cookie->max_len))
+  if (!((strlen(cookie->data) + strlen(cook)) < cookie->max_len))
     return -1;                  // impossible d'ajouter
   cookie_insert(insert, cook);
 #if DEBUG_COOK
@@ -111,13 +112,13 @@ int cookie_add(t_cookie * cookie, char *cook_name, char *cook_value,
 }
 
 // effacer cookie si existe
-int cookie_del(t_cookie * cookie, char *cook_name, char *domain, char *path) {
+int cookie_del(t_cookie * cookie, const char *cook_name, const char *domain, const char *path) {
   char *a, *b;
 
   b = cookie_find(cookie->data, cook_name, domain, path);
   if (b) {
     a = cookie_nextfield(b);
-    cookie_delete(b, (int) (a - b));
+    cookie_delete(b, a - b);
 #if DEBUG_COOK
     printf("deleted old cookie: %s %s %s\n", cook_name, domain, path);
 #endif
@@ -128,11 +129,11 @@ int cookie_del(t_cookie * cookie, char *cook_name, char *domain, char *path) {
 // Matches wildcard cookie domains that start with a dot
 // chk_dom: the domain stored in the cookie (potentially wildcard).
 // domain: query domain
-static int cookie_cmp_wildcard_domain(char *chk_dom, char *domain) {
-  int n = strlen(chk_dom);
-  int m = strlen(domain);
-  int l = n < m ? n : m;
-  int i;
+static int cookie_cmp_wildcard_domain(const char *chk_dom, const char *domain) {
+  const size_t n = strlen(chk_dom);
+  const size_t m = strlen(domain);
+  const size_t l = n < m ? n : m;
+  size_t i;
   for (i = l - 1; i >= 0; i--) {
     if (chk_dom[n - i - 1] != domain[m - i - 1]) {
       return 1;
@@ -152,7 +153,7 @@ static int cookie_cmp_wildcard_domain(char *chk_dom, char *domain) {
 // renvoie pointeur sur ligne, ou NULL si introuvable
 // path est aligné à droite et cook_name peut être vide (chercher alors tout cookie)
 // .doubleclick.net     TRUE    /       FALSE   1999999999      id      A
-char *cookie_find(char *s, char *cook_name, char *domain, char *path) {
+char *cookie_find(char *s, const char *cook_name, const char *domain, const char *path) {
   char buffer[8192];
   char *a = s;
 
@@ -165,15 +166,15 @@ char *cookie_find(char *s, char *cook_name, char *domain, char *path) {
       t = (strcmp(cookie_get(buffer, a, 5), cook_name) == 0);   // tester si même nom
     if (t) {                    // même nom ou nom qualconque
       //
-      char *chk_dom = cookie_get(buffer, a, 0); // domaine concerné par le cookie
+      const char *chk_dom = cookie_get(buffer, a, 0); // domaine concerné par le cookie
 
-      if (((int) strlen(chk_dom) <= (int) strlen(domain) &&
+      if ((strlen(chk_dom) <= strlen(domain) &&
         strcmp(chk_dom, domain + strlen(domain) - strlen(chk_dom)) == 0) ||
         !cookie_cmp_wildcard_domain(chk_dom, domain)) {  // même domaine
           //
-        char *chk_path = cookie_get(buffer, a, 2);    // chemin concerné par le cookie
+        const char *chk_path = cookie_get(buffer, a, 2);    // chemin concerné par le cookie
 
-        if ((int) strlen(chk_path) <= (int) strlen(path)) {
+        if (strlen(chk_path) <= strlen(path)) {
           if (strncmp(path, chk_path, strlen(chk_path)) == 0) {       // même chemin
             return a;
           }
@@ -222,7 +223,7 @@ int cookie_load(t_cookie * cookie, const char *fpath, const char *name) {
       do {
         if (!(find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
           if (!(find.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM)) {
-            FILE *fp = fopen(fconcat(catbuff, fpath, find.cFileName), "rb");
+            FILE *fp = fopen(fconcat(catbuff, sizeof(catbuff), fpath, find.cFileName), "rb");
 
             if (fp) {
               char cook_name[256];
@@ -231,15 +232,14 @@ int cookie_load(t_cookie * cookie, const char *fpath, const char *name) {
               char dummy[512];
 
               //
-              char domain[256]; // domaine cookie (.netscape.com)
-              char path[256];   // chemin (/)
+              lien_adrfil af;   // chemin (/)
               int cookie_merged = 0;
 
               //
               // Read all cookies
               while(!feof(fp)) {
                 cook_name[0] = cook_value[0] = domainpathpath[0]
-                  = dummy[0] = domain[0] = path[0] = '\0';
+                = dummy[0] = af.adr[0] = af.fil[0] = '\0';
                 linput(fp, cook_name, 250);
                 if (!feof(fp)) {
                   linput(fp, cook_value, 250);
@@ -254,8 +254,8 @@ int cookie_load(t_cookie * cookie, const char *fpath, const char *name) {
                     if (strnotempty(cook_name)
                         && strnotempty(cook_value)
                         && strnotempty(domainpathpath)) {
-                      if (ident_url_absolute(domainpathpath, domain, path) >= 0) {
-                        cookie_add(cookie, cook_name, cook_value, domain, path);
+                      if (ident_url_absolute(domainpathpath, &af) >= 0) {
+                        cookie_add(cookie, cook_name, cook_value, af.adr, af.fil);
                         cookie_merged = 1;
                       }
                     }
@@ -264,7 +264,7 @@ int cookie_load(t_cookie * cookie, const char *fpath, const char *name) {
               }
               fclose(fp);
               if (cookie_merged)
-                remove(fconcat(catbuff, fpath, find.cFileName));
+                remove(fconcat(catbuff, sizeof(catbuff), fpath, find.cFileName));
             }                   // if fp
           }
       } while(FindNextFileA(h, &find));
@@ -275,7 +275,7 @@ int cookie_load(t_cookie * cookie, const char *fpath, const char *name) {
 
   // Ensuite, cookies.txt
   {
-    FILE *fp = fopen(fconcat(catbuff, fpath, name), "rb");
+    FILE *fp = fopen(fconcat(catbuff, sizeof(catbuff), fpath, name), "rb");
 
     if (fp) {
       char BIGSTK line[8192];
@@ -311,12 +311,12 @@ int cookie_load(t_cookie * cookie, const char *fpath, const char *name) {
 
 // écrire cookies.txt
 // !=0 : erreur
-int cookie_save(t_cookie * cookie, char *name) {
+int cookie_save(t_cookie * cookie, const char *name) {
   char catbuff[CATBUFF_SIZE];
 
   if (strnotempty(cookie->data)) {
     char BIGSTK line[8192];
-    FILE *fp = fopen(fconv(catbuff, name), "wb");
+    FILE *fp = fopen(fconv(catbuff, sizeof(catbuff), name), "wb");
 
     if (fp) {
       char *a = cookie->data;
@@ -337,13 +337,13 @@ int cookie_save(t_cookie * cookie, char *name) {
 }
 
 // insertion chaine ins avant s
-void cookie_insert(char *s, char *ins) {
+void cookie_insert(char *s, const char *ins) {
   char *buff;
 
   if (strnotempty(s) == 0) {    // rien à faire, juste concat
     strcatbuff(s, ins);
   } else {
-    buff = (char *) malloct(strlen(s) + 2);
+    buff = (char *) malloct(strlen(s) + 1);
     if (buff) {
       strcpybuff(buff, s);      // copie temporaire
       strcpybuff(s, ins);       // insérer
@@ -354,13 +354,13 @@ void cookie_insert(char *s, char *ins) {
 }
 
 // destruction chaine dans s position pos
-void cookie_delete(char *s, int pos) {
+void cookie_delete(char *s, size_t pos) {
   char *buff;
 
   if (strnotempty(s + pos) == 0) {      // rien à faire, effacer
     s[0] = '\0';
   } else {
-    buff = (char *) malloct(strlen(s + pos) + 2);
+    buff = (char *) malloct(strlen(s + pos) + 1);
     if (buff) {
       strcpybuff(buff, s + pos);        // copie temporaire
       strcpybuff(s, buff);      // copier
@@ -371,8 +371,8 @@ void cookie_delete(char *s, int pos) {
 
 // renvoie champ param de la chaine cookie_base
 // ex: cookie_get("ceci est<tab>un<tab>exemple",1) renvoi "un"
-char *cookie_get(char *buffer, char *cookie_base, int param) {
-  char *limit;
+const char *cookie_get(char *buffer, const char *cookie_base, int param) {
+  const char *limit;
 
   while(*cookie_base == '\n')
     cookie_base++;
@@ -393,7 +393,7 @@ char *cookie_get(char *buffer, char *cookie_base, int param) {
     }
     if (cookie_base) {
       if (cookie_base < limit) {
-        char *a = cookie_base;
+        const char *a = cookie_base;
 
         while((*a) && (*a != '\t') && (*a != '\n'))
           a++;
@@ -413,7 +413,7 @@ char *cookie_get(char *buffer, char *cookie_base, int param) {
 // -- basic auth --
 
 /* déclarer un répertoire comme possédant une authentification propre */
-int bauth_add(t_cookie * cookie, char *adr, char *fil, char *auth) {
+int bauth_add(t_cookie * cookie, const char *adr, const char *fil, const char *auth) {
   char buffer[HTS_URLMAXSIZE * 2];
 
   if (cookie) {
@@ -439,7 +439,7 @@ int bauth_add(t_cookie * cookie, char *adr, char *fil, char *auth) {
 
 /* tester adr et fil, et retourner authentification si nécessaire */
 /* sinon, retourne NULL */
-char *bauth_check(t_cookie * cookie, char *adr, char *fil) {
+char *bauth_check(t_cookie * cookie, const char *adr, const char *fil) {
   char buffer[HTS_URLMAXSIZE * 2];
 
   if (cookie) {
@@ -458,10 +458,10 @@ char *bauth_check(t_cookie * cookie, char *adr, char *fil) {
   return NULL;
 }
 
-char *bauth_prefix(char *prefix, char *adr, char *fil) {
+char *bauth_prefix(char *prefix, const char *adr, const char *fil) {
   char *a;
 
-  strcpybuff(prefix, jump_identification(adr));
+  strcpybuff(prefix, jump_identification_const(adr));
   strcatbuff(prefix, fil);
   a = strchr(prefix, '?');
   if (a)
