@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -231,8 +231,15 @@ struct t_hts_callback_ref {
   size_t offset;
 };
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern const t_hts_htmlcheck_callbacks default_callbacks;
-extern const t_hts_callback_ref default_callbacks_ref[];
+
+#ifdef __cplusplus
+}
+#endif
 
 #define HT_PRINT(A) strcatbuff(opt->state.HTbuff,A);
 #define HT_REQUEST_START opt->state.HTbuff[0]='\0';
