@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -39,11 +39,11 @@ Please visit our Website: http://www.httrack.com
 
 #include "htsbase.h"
 
-int fa_strjoker(int type, char **filters, int nfil, char *nom, LLint * size,
+int fa_strjoker(int type, char **filters, int nfil, const char *nom, LLint * size,
                 int *size_flag, int *depth);
-HTS_INLINE char *strjoker(char *chaine, char *joker, LLint * size,
+HTS_INLINE const char *strjoker(const char *chaine, const char *joker, LLint * size,
                           int *size_flag);
-char *strjokerfind(char *chaine, char *joker);
+const char *strjokerfind(const char *chaine, const char *joker);
 #endif
 
 #endif
