@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -34,15 +34,15 @@ Please visit our Website: http://www.httrack.com
 /* Internal engine bytecode */
 #define HTS_INTERNAL_BYTECODE
 
+#include "htsopt.h"
 #include "htshash.h"
 
 /* specific definitions */
 #include "htsbase.h"
-#include "htsopt.h"
 #include "htsglobal.h"
 #include "htsmd5.h"
 #include "htscore.h"
-#include "htsinthash.h"
+#include "coucal.h"
 /* END specific definitions */
 
 /* Specific macros */
@@ -60,40 +60,55 @@ Please visit our Website: http://www.httrack.com
 // recherche dans la table selon nom1,nom2 et le no d'enregistrement
 
 /* Key free handler (NOOP) ; addresses are kept */
-static void key_freehandler(void *arg, void *value) {
+static void key_freehandler(void *arg, coucal_key value) {
 }
 
 /* Key strdup (pointer copy) */
-static char* key_duphandler(void *arg, const char *name) {
+static coucal_key key_duphandler(void *arg, coucal_key_const name) {
   union {
-    const char *roname;
-    char *name;
+    coucal_key_const roname;
+    coucal_key name;
   } u;
   u.roname = name;
   return u.name;
 }
 
 /* Key sav hashes are using case-insensitive version */
-static inthash_keys key_sav_hashes(void *arg, const char *value) {
+static coucal_hashkeys key_sav_hashes(void *arg, coucal_key_const key) {
   hash_struct *const hash = (hash_struct*) arg;
-  convtolower(hash->catbuff, value);
-  return inthash_hash_value(hash->catbuff);
+  convtolower(hash->catbuff, (const char*) key);
+  return coucal_hash_string(hash->catbuff);
 }
 
 /* Key sav comparison is case-insensitive */
-static int key_sav_equals(void *arg, const char *a, const char *b) {
+static int key_sav_equals(void *arg,
+                          coucal_key_const a_,
+                          coucal_key_const b_) {
+  const char *const a = (const char*) a_;
+  const char *const b = (const char*) b_;
   return strcasecmp(a, b) == 0;
 }
 
+static const char* key_sav_debug_print(void *arg,
+                                       coucal_key_const a) {
+  return (const char*) a;
+}
+
+static const char* value_sav_debug_print(void *arg, coucal_value_const a) {
+  return (char*) a.ptr;
+}
+
 /* Pseudo-key (lien_url structure) hash function */
-static inthash_keys key_adrfil_hashes_generic(void *arg, const char *value_, 
+static coucal_hashkeys key_adrfil_hashes_generic(void *arg,
+                                              coucal_key_const value, 
                                               const int former) {
   hash_struct *const hash = (hash_struct*) arg;
-  const lien_url*const lien = (lien_url*) value_;
+  const lien_url*const lien = (const lien_url*) value;
   const char *const adr = !former ? lien->adr : lien->former_adr;
   const char *const fil = !former ? lien->fil : lien->former_fil;
   const char *const adr_norm = adr != NULL ? 
-    ( hash->normalized  ? jump_normalized(adr) : jump_identification(adr) )
+    ( hash->normalized  ? jump_normalized_const(adr)
+                        : jump_identification_const(adr) )
     : NULL;
 
   // copy address
@@ -109,16 +124,18 @@ static inthash_keys key_adrfil_hashes_generic(void *arg, const char *value_,
   }
 
   // hash
-  return inthash_hash_value(hash->normfil);
+  return coucal_hash_string(hash->normfil);
 }
 
 /* Pseudo-key (lien_url structure) comparison function */
-static int key_adrfil_equals_generic(void *arg, const char *a_, const char *b_, 
+static int key_adrfil_equals_generic(void *arg,
+                                     coucal_key_const a_,
+                                     coucal_key_const b_, 
                                      const int former) {
   hash_struct *const hash = (hash_struct*) arg;
   const int normalized = hash->normalized;
-  const lien_url*const a = (lien_url*) a_;
-  const lien_url*const b = (lien_url*) b_;
+  const lien_url*const a = (const lien_url*) a_;
+  const lien_url*const b = (const lien_url*) b_;
   const char *const a_adr = !former ? a->adr : a->former_adr;
   const char *const b_adr = !former ? b->adr : b->former_adr;
   const char *const a_fil = !former ? a->fil : a->former_fil;
@@ -133,8 +150,10 @@ static int key_adrfil_equals_generic(void *arg, const char *a_, const char *b_,
   assertf(b_fil != NULL);
 
   // skip scheme and authentication to the domain (possibly without www.)
-  ja = normalized ? jump_normalized(a_adr) : jump_identification(a_adr);
-  jb = normalized ? jump_normalized(b_adr) : jump_identification(b_adr);
+  ja = normalized
+    ? jump_normalized_const(a_adr) : jump_identification_const(a_adr);
+  jb = normalized
+    ? jump_normalized_const(b_adr) : jump_identification_const(b_adr);
   assertf(ja != NULL);
   assertf(jb != NULL);
   if (strcasecmp(ja, jb) != 0) {
@@ -151,34 +170,74 @@ static int key_adrfil_equals_generic(void *arg, const char *a_, const char *b_,
   }
 }
 
+static const char* key_adrfil_debug_print_(void *arg,
+                                           coucal_key_const a_,
+                                           const int former) {
+  hash_struct *const hash = (hash_struct*) arg;
+  const lien_url*const a = (const lien_url*) a_;
+  const char *const a_adr = !former ? a->adr : a->former_adr;
+  const char *const a_fil = !former ? a->fil : a->former_fil;
+  snprintf(hash->normfil, sizeof(hash->normfil), "%s%s", a_adr, a_fil);
+  return hash->normfil;
+}
+
+static const char* key_adrfil_debug_print(void *arg,
+                                          coucal_key_const a_) {
+  return key_adrfil_debug_print_(arg, a_, 0);
+}
+
+static const char* key_former_adrfil_debug_print(void *arg,
+                                                 coucal_key_const a_) {
+  return key_adrfil_debug_print_(arg, a_, 1);
+}
+
+static const char* value_adrfil_debug_print(void *arg,
+                                            coucal_value_const value) {
+  hash_struct *const hash = (hash_struct*) arg;
+  snprintf(hash->normfil2, sizeof(hash->normfil2), "%d", (int) value.intg);
+  return hash->normfil2;
+}
+
 /* "adr"/"fil" lien_url structure members hashing function */
-static inthash_keys key_adrfil_hashes(void *arg, const char *value_) {
+static coucal_hashkeys key_adrfil_hashes(void *arg, coucal_key_const value_) {
   return key_adrfil_hashes_generic(arg, value_, 0);
 }
 
 /* "adr"/"fil" lien_url structure members comparison function */
-static int key_adrfil_equals(void *arg, const char *a, const char *b) {
+static int key_adrfil_equals(void *arg,
+                             coucal_key_const a,
+                             coucal_key_const b) {
   return key_adrfil_equals_generic(arg, a, b, 0);
 }
 
 /* "former_adr"/"former_fil" lien_url structure members hashing function */
-static inthash_keys key_former_adrfil_hashes(void *arg, const char *value_) {
+static coucal_hashkeys key_former_adrfil_hashes(void *arg, coucal_key_const value_) {
   return key_adrfil_hashes_generic(arg, value_, 1);
 }
 
 /* "former_adr"/"former_fil" lien_url structure members comparison function */
-static int key_former_adrfil_equals(void *arg, const char *a, const char *b) {
+static int key_former_adrfil_equals(void *arg,
+                                    coucal_key_const a,
+                                    coucal_key_const b) {
   return key_adrfil_equals_generic(arg, a, b, 1);
 }
 
-void hash_init(hash_struct * hash, int normalized) {
-  hash->sav = inthash_new(0);
-  hash->adrfil = inthash_new(0);
-  hash->former_adrfil = inthash_new(0);
+void hash_init(httrackp *opt, hash_struct * hash, int normalized) {
+  hash->sav = coucal_new(0);
+  hash->adrfil = coucal_new(0);
+  hash->former_adrfil = coucal_new(0);
   hash->normalized = normalized;
 
+  hts_set_hash_handler(hash->sav, opt);
+  hts_set_hash_handler(hash->adrfil, opt);
+  hts_set_hash_handler(hash->former_adrfil, opt);
+
+  coucal_set_name(hash->sav, "hash->sav");
+  coucal_set_name(hash->adrfil, "hash->adrfil");
+  coucal_set_name(hash->former_adrfil, "hash->former_adrfil");
+
   /* Case-insensitive comparison ; keys are direct char* filenames */
-  inthash_value_set_key_handler(hash->sav,
+  coucal_value_set_key_handler(hash->sav,
                                 key_duphandler,
                                 key_freehandler,
                                 key_sav_hashes,
@@ -187,25 +246,39 @@ void hash_init(hash_struct * hash, int normalized) {
 
   /* URL-style comparison ; keys are lien_url structure pointers casted 
      to char* */
-  inthash_value_set_key_handler(hash->adrfil,
+  coucal_value_set_key_handler(hash->adrfil,
                                 key_duphandler,
                                 key_freehandler,
                                 key_adrfil_hashes,
                                 key_adrfil_equals,
                                 hash);
-  inthash_value_set_key_handler(hash->former_adrfil,
+  coucal_value_set_key_handler(hash->former_adrfil,
                                 key_duphandler,
                                 key_freehandler,
                                 key_former_adrfil_hashes,
                                 key_former_adrfil_equals,
                                 hash);
+
+  /* pretty-printing */
+  coucal_set_print_handler(hash->sav,
+                            key_sav_debug_print,
+                            value_sav_debug_print,
+                            NULL);
+  coucal_set_print_handler(hash->adrfil,
+                            key_adrfil_debug_print,
+                            value_adrfil_debug_print,
+                            hash);
+  coucal_set_print_handler(hash->former_adrfil,
+                            key_former_adrfil_debug_print,
+                            value_adrfil_debug_print,
+                            hash);
 }
 
 void hash_free(hash_struct *hash) {
   if (hash != NULL) {
-    inthash_delete(&hash->sav);
-    inthash_delete(&hash->adrfil);
-    inthash_delete(&hash->former_adrfil);
+    coucal_delete(&hash->sav);
+    coucal_delete(&hash->adrfil);
+    coucal_delete(&hash->former_adrfil);
   }
 }
 
@@ -218,7 +291,7 @@ int hash_read(const hash_struct * hash, const char *nom1, const char *nom2,
   /* read */
   switch(type) {
   case HASH_STRUCT_FILENAME:
-    if (inthash_read(hash->sav, nom1, &intvalue)) {
+    if (coucal_read(hash->sav, nom1, &intvalue)) {
       return (int) intvalue;
     } else {
       return -1;
@@ -228,7 +301,7 @@ int hash_read(const hash_struct * hash, const char *nom1, const char *nom2,
     memset(&lien, 0, sizeof(lien));
     lien.adr = key_duphandler(NULL, nom1);
     lien.fil = key_duphandler(NULL, nom2);
-    if (inthash_read(hash->adrfil, (char*) &lien, &intvalue)) {
+    if (coucal_read(hash->adrfil, (char*) &lien, &intvalue)) {
       return (int) intvalue;
     } else {
       return -1;
@@ -238,7 +311,7 @@ int hash_read(const hash_struct * hash, const char *nom1, const char *nom2,
     memset(&lien, 0, sizeof(lien));
     lien.former_adr = key_duphandler(NULL, nom1);
     lien.former_fil = key_duphandler(NULL, nom2);
-    if (inthash_read(hash->former_adrfil, (char*) &lien, &intvalue)) {
+    if (coucal_read(hash->former_adrfil, (char*) &lien, &intvalue)) {
       return (int) intvalue;
     } else {
       return -1;
@@ -252,15 +325,15 @@ int hash_read(const hash_struct * hash, const char *nom1, const char *nom2,
 }
 
 // enregistrement lien lpos dans les 3 tables hash1..3
-void hash_write(hash_struct * hash, int lpos) {
+void hash_write(hash_struct * hash, size_t lpos) {
   /* first entry: destination filename (lowercased) */
-  inthash_write(hash->sav, hash->liens[lpos]->sav, lpos);
+  coucal_write(hash->sav, (*hash->liens)[lpos]->sav, lpos);
 
   /* second entry: URL address and path */
-  inthash_write(hash->adrfil, (char*) hash->liens[lpos], lpos);
+  coucal_write(hash->adrfil, (*hash->liens)[lpos], lpos);
 
   /* third entry: URL address and path before redirect */
-  if (hash->liens[lpos]->former_adr) {        // former_adr existe?
-    inthash_write(hash->former_adrfil, (char*) hash->liens[lpos], lpos);
+  if ((*hash->liens)[lpos]->former_adr) {        // former_adr existe?
+    coucal_write(hash->former_adrfil, (*hash->liens)[lpos], lpos);
   }
 }
