@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -43,11 +43,11 @@ Please visit our Website: http://www.httrack.com
 typedef struct httrackp httrackp;
 #endif
 
-void infomsg(char *msg);
-void help(char *app, int more);
-void make_empty_index(char *str);
+void infomsg(const char *msg);
+void help(const char *app, int more);
+void make_empty_index(const char *str);
 void help_wizard(httrackp * opt);
-int help_query(char *list, int def);
+int help_query(const char *list, int def);
 void help_catchurl(const char *dest_path);
 
 #endif
