@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ Please visit our Website: http://www.httrack.com
 
 #if HTS_MAKE_KEYWORD_INDEX
 #include "htshash.h"
-#include "htsinthash.h"
+#include "coucal.h"
 
 /* Keyword Indexer Parameters */
 
@@ -103,7 +103,7 @@ Please visit our Website: http://www.httrack.com
 
 /* End of Keyword Indexer Parameters */
 
-int strcpos(char *adr, char c);
+int strcpos(const char *adr, char c);
 int mystrcmp(const void *_e1, const void *_e2);
 
 // Global variables
@@ -144,7 +144,7 @@ int index_keyword(const char *html_data, LLint size, const char *mime,
 
   //
   //int WordIndexSize = 1024;
-  inthash WordIndexHash = NULL;
+  coucal WordIndexHash = NULL;
   FILE *tmpfp = NULL;
 
   //
@@ -161,8 +161,8 @@ int index_keyword(const char *html_data, LLint size, const char *mime,
 
   // Init ?
   if (hts_index_init) {
-    UNLINK(concat(catbuff, indexpath, "index.txt"));
-    UNLINK(concat(catbuff, indexpath, "sindex.html"));
+    UNLINK(concat(catbuff, sizeof(catbuff), indexpath, "index.txt"));
+    UNLINK(concat(catbuff, sizeof(catbuff), indexpath, "sindex.html"));
     hts_index_init = 0;
   }
   // Check MIME type
@@ -190,7 +190,7 @@ int index_keyword(const char *html_data, LLint size, const char *mime,
 
   // Create hash structure
   // Hash tables rulez da world!
-  WordIndexHash = inthash_new(0);
+  WordIndexHash = coucal_new(0);
   if (!WordIndexHash)
     return 0;
 
@@ -252,7 +252,7 @@ int index_keyword(const char *html_data, LLint size, const char *mime,
           /* Store it ? */
           if (len >= KEYW_MIN_LEN) {
             hts_primindex_words++;
-            if (inthash_inc(WordIndexHash, keyword)) {  /* added new */
+            if (coucal_inc(WordIndexHash, keyword)) {  /* added new */
               fprintf(tmpfp, "%s\n", keyword);
             }
           }
@@ -285,13 +285,13 @@ int index_keyword(const char *html_data, LLint size, const char *mime,
         if (strnotempty(line)) {
           intptr_t e = 0;
 
-          if (inthash_read(WordIndexHash, line, &e)) {
+          if (coucal_read(WordIndexHash, line, &e)) {
             //if (e) {
             char BIGSTK savelst[HTS_URLMAXSIZE * 2];
 
             e++;                /* 0 means "once" */
 
-            if (strncmp((const char *) fslash(catbuff, (char *) indexpath), filename, strlen(indexpath)) == 0)  // couper
+            if (strncmp((const char *) fslash(catbuff, sizeof(catbuff), (const char *) indexpath), filename, strlen(indexpath)) == 0)  // couper
               strcpybuff(savelst, filename + strlen(indexpath));
             else
               strcpybuff(savelst, filename);
@@ -313,7 +313,7 @@ int index_keyword(const char *html_data, LLint size, const char *mime,
   tmpfp = NULL;
 
   // Clear hash table
-  inthash_delete(&WordIndexHash);
+  coucal_delete(&WordIndexHash);
 #endif
   return 1;
 }
@@ -358,9 +358,9 @@ void index_finish(const char *indexpath, int mode) {
 
             // Write new file
             if (mode == 1)      // TEXT
-              fp = fopen(concat(catbuff, indexpath, "index.txt"), "wb");
+              fp = fopen(concat(catbuff, sizeof(catbuff), indexpath, "index.txt"), "wb");
             else                // HTML
-              fp = fopen(concat(catbuff, indexpath, "sindex.html"), "wb");
+              fp = fopen(concat(catbuff, sizeof(catbuff), indexpath, "sindex.html"), "wb");
             if (fp) {
               char current_word[KEYW_LEN + 32];
               char word[KEYW_LEN + 32];
@@ -468,8 +468,8 @@ void index_finish(const char *indexpath, int mode) {
 /* Subroutines */
 
 #if HTS_MAKE_KEYWORD_INDEX
-int strcpos(char *adr, char c) {
-  char *apos = strchr(adr, c);
+int strcpos(const char *adr, char c) {
+  const char *apos = strchr(adr, c);
 
   if (apos)
     return (int) (apos - adr);
@@ -478,8 +478,8 @@ int strcpos(char *adr, char c) {
 }
 
 int mystrcmp(const void *_e1, const void *_e2) {
-  char **e1 = (char **) _e1;
-  char **e2 = (char **) _e2;
+  const char *const*const e1 = (const char *const*) _e1;
+  const char *const*const e2 = (const char *const*) _e2;
 
   return strcmp(*e1, *e2);
 }
