@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -89,30 +89,6 @@ HTSEXT_API const char *hts_get_version_info(httrackp * opt) {
   return opt->state.HTbuff;
 }
 
-/* memory checks */
-HTSEXT_API htsErrorCallback htsCallbackErr = NULL;
-HTSEXT_API int htsMemoryFastXfr = 1;    /* fast xfr by default */
-void abortLog__fnc(char *msg, char *file, int line);
-void abortLog__fnc(char *msg, char *file, int line) {
-  FILE *fp = fopen("CRASH.TXT", "wb");
-
-  if (!fp)
-    fp = fopen("/tmp/CRASH.TXT", "wb");
-  if (!fp)
-    fp = fopen("C:\\CRASH.TXT", "wb");
-  if (!fp)
-    fp = fopen("CRASH.TXT", "wb");
-  if (fp) {
-    fprintf(fp, "HTTrack " HTTRACK_VERSIONID " closed at '%s', line %d\r\n",
-            file, line);
-    fprintf(fp, "Reason:\r\n%s\r\n", msg);
-    fflush(fp);
-    fclose(fp);
-  }
-}
-
-HTSEXT_API t_abortLog abortLog__ = abortLog__fnc;       /* avoid VC++ inlining */
-
 static void htspe_log(htsmoduleStruct * str, const char *msg);
 
 int hts_parse_externals(htsmoduleStruct * str) {
@@ -242,17 +218,17 @@ void htspe_init(void) {
     /* See CVE-2010-5252 */
 #if (defined(_WIN32) && (!defined(_DEBUG)))
     {
-      /* >= Windows Server 2003 (Andy Hewitt) */
-      const DWORD dwVersion = GetVersion();
-      const DWORD dwMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
-      const DWORD dwMinorVersion = (DWORD)(HIBYTE(LOWORD(dwVersion)));
-      const int hasSetDllDirectory =
-        dwMajorVersion >= 6 || ( dwMajorVersion == 5 && dwMinorVersion >= 2 );
       /* See KB 2389418
          "If this parameter is an empty string (""), the call removes the 
          current directory from the default DLL search order" */
-      if (hasSetDllDirectory && !SetDllDirectory("")) {
+      BOOL (WINAPI*const k32_SetDllDirectoryA)(LPCSTR) = 
+        (BOOL (WINAPI *)(LPCSTR))
+        GetProcAddress(GetModuleHandle("kernel32.dll"), "SetDllDirectoryA");
+      if (k32_SetDllDirectoryA != NULL && !k32_SetDllDirectoryA("")) {
         /* Do no choke on NT or 98SE with KernelEx NT API (James Blough) */
+        const DWORD dwVersion = GetVersion();
+        const DWORD dwMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
+        const DWORD dwMinorVersion = (DWORD)(HIBYTE(LOWORD(dwVersion)));
         if (dwMajorVersion >= 5) {
           assertf(!"SetDllDirectory failed");
         }
