@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ Please visit our Website: http://www.httrack.com
 // -- robots --
 
 // fil="" : vérifier si règle déja enregistrée
-int checkrobots(robots_wizard * robots, char *adr, char *fil) {
+int checkrobots(robots_wizard * robots, const char *adr, const char *fil) {
   while(robots) {
     if (strfield2(robots->adr, adr)) {
       if (fil[0]) {
@@ -74,7 +74,7 @@ int checkrobots(robots_wizard * robots, char *adr, char *fil) {
   }
   return 0;
 }
-int checkrobots_set(robots_wizard * robots, char *adr, char *data) {
+int checkrobots_set(robots_wizard * robots, const char *adr, const char *data) {
   if (((int) strlen(adr)) >= sizeof(robots->adr) - 2)
     return 0;
   if (((int) strlen(data)) >= sizeof(robots->token) - 2)
