@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -68,16 +68,16 @@ struct find_handle_struct {
   char path[2048];
 };
 #endif
-#ifndef HTS_DEF_FWSTRUCT_topindex_chain
-#define HTS_DEF_FWSTRUCT_topindex_chain
-typedef struct topindex_chain topindex_chain;
-#endif
-struct topindex_chain {
-  int level;                    /* sort level */
-  char *category;               /* category */
-  char name[2048];              /* path */
-  struct topindex_chain *next;  /* next element */
-};
+//#ifndef HTS_DEF_FWSTRUCT_topindex_chain
+//#define HTS_DEF_FWSTRUCT_topindex_chain
+//typedef struct topindex_chain topindex_chain;
+//#endif
+//struct topindex_chain {
+//  int level;                    /* sort level */
+//  char *category;               /* category */
+//  char name[2048];              /* path */
+//  struct topindex_chain *next;  /* next element */
+//};
 
 /* Tools */
 
@@ -91,11 +91,11 @@ static int ehexh(char c) {
   return 0;
 }
 
-static int ehex(char *s) {
+static int ehex(const char *s) {
   return 16 * ehexh(*s) + ehexh(*(s + 1));
 }
 
-static void unescapehttp(char *s, String * tempo) {
+static void unescapehttp(const char *s, String * tempo) {
   size_t i;
 
   for(i = 0; s[i] != '\0'; i++) {
@@ -122,12 +122,15 @@ static void unescapehttp(char *s, String * tempo) {
 // -1 : erreur
 // -2 : protocole non supporté (ftp)
 int ident_url_relatif(const char *lien, const char *origin_adr,
-                      const char *origin_fil, char *adr, char *fil) {
+                      const char *origin_fil,
+                      lien_adrfil* const adrfil) {
   int ok = 0;
   int scheme = 0;
 
-  adr[0] = '\0';
-  fil[0] = '\0';                //effacer buffers
+  assertf(adrfil != NULL);
+
+  adrfil->adr[0] = '\0';
+  adrfil->fil[0] = '\0';                //effacer buffers
 
   // lien non vide!
   if (strnotempty(lien) == 0)
@@ -149,13 +152,13 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
       || (strfield(lien, "file://"))    // scheme+//
       || (strncmp(lien, "//", 2) == 0)  // // sans scheme (-> default)
     ) {
-    if (ident_url_absolute(lien, adr, fil) == -1) {
+    if (ident_url_absolute(lien, adrfil) == -1) {
       ok = -1;                  // erreur URL
     }
   } else if (strfield(lien, "ftp://")) {
     // Note: ftp:foobar.gif is not valid
     if (ftp_available()) {      // ftp supporté
-      if (ident_url_absolute(lien, adr, fil) == -1) {
+      if (ident_url_absolute(lien, adrfil) == -1) {
         ok = -1;                // erreur URL
       }
     } else {
@@ -164,7 +167,7 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
 #if HTS_USEOPENSSL
   } else if (strfield(lien, "https://")) {
     // Note: ftp:foobar.gif is not valid
-    if (ident_url_absolute(lien, adr, fil) == -1) {
+    if (ident_url_absolute(lien, adrfil) == -1) {
       ok = -1;                // erreur URL
     }
 #endif
@@ -191,30 +194,30 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
       /* patch scheme if necessary */
       if (strfield(lien, "http:")) {
         lien += 5;
-        strcpybuff(adr, jump_protocol(origin_adr));     // même adresse ; protocole vide (http)
+        strcpybuff(adrfil->adr, jump_protocol_const(origin_adr));     // même adresse ; protocole vide (http)
       } else if (strfield(lien, "https:")) {
         lien += 6;
-        strcpybuff(adr, "https://");    // même adresse forcée en https
-        strcatbuff(adr, jump_protocol(origin_adr));
+        strcpybuff(adrfil->adr, "https://");    // même adresse forcée en https
+        strcatbuff(adrfil->adr, jump_protocol_const(origin_adr));
       } else if (strfield(lien, "ftp:")) {
         lien += 4;
-        strcpybuff(adr, "ftp://");      // même adresse forcée en ftp
-        strcatbuff(adr, jump_protocol(origin_adr));
+        strcpybuff(adrfil->adr, "ftp://");      // même adresse forcée en ftp
+        strcatbuff(adrfil->adr, jump_protocol_const(origin_adr));
       } else {
-        strcpybuff(adr, origin_adr);    // même adresse ; et même éventuel protocole
+        strcpybuff(adrfil->adr, origin_adr);    // même adresse ; et même éventuel protocole
       }
 
       if (*lien != '/') {       // sinon c'est un lien absolu
         if (*lien == '\0') {
-          strcpybuff(fil, origin_fil);
+          strcpybuff(adrfil->fil, origin_fil);
         } else if (*lien == '?') {      // example: a href="?page=2"
           char *a;
 
-          strcpybuff(fil, origin_fil);
-          a = strchr(fil, '?');
+          strcpybuff(adrfil->fil, origin_fil);
+          a = strchr(adrfil->fil, '?');
           if (a)
             *a = '\0';
-          strcatbuff(fil, lien);
+          strcatbuff(adrfil->fil, lien);
         } else {
           const char *a = strchr(origin_fil, '?');
 
@@ -225,14 +228,14 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
           if (*a == '/') {      // ok on a un '/'
             if ((((int) (a - origin_fil)) + 1 + strlen(lien)) < HTS_URLMAXSIZE) {
               // copier chemin
-              strncpy(fil, origin_fil, ((int) (a - origin_fil)) + 1);
-              *(fil + ((int) (a - origin_fil)) + 1) = '\0';
+              strncpy(adrfil->fil, origin_fil, ((int) (a - origin_fil)) + 1);
+              *(adrfil->fil + ((int) (a - origin_fil)) + 1) = '\0';
 
               // copier chemin relatif
-              if (((int) strlen(fil) + (int) strlen(lien)) < HTS_URLMAXSIZE) {
-                strcatbuff(fil, lien + ((*lien == '/') ? 1 : 0));
+              if (((int) strlen(adrfil->fil) + (int) strlen(lien)) < HTS_URLMAXSIZE) {
+                strcatbuff(adrfil->fil, lien + ((*lien == '/') ? 1 : 0));
                 // simplifier url pour les ../
-                fil_simplifie(fil);
+                fil_simplifie(adrfil->fil);
               } else
                 ok = -1;        // erreur
             } else {            // erreur
@@ -244,8 +247,8 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
         }
       } else {                  // chemin absolu
         // copier chemin directement
-        strcatbuff(fil, lien);
-        fil_simplifie(fil);
+        strcatbuff(adrfil->fil, lien);
+        fil_simplifie(adrfil->fil);
       }                         // *lien!='/'
     } else
       ok = -1;
@@ -254,7 +257,7 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
 
   // case insensitive pour adresse
   {
-    char *a = jump_identification(adr);
+    char *a = jump_identification(adrfil->adr);
 
     while(*a) {
       if ((*a >= 'A') && (*a <= 'Z'))
@@ -264,8 +267,8 @@ int ident_url_relatif(const char *lien, const char *origin_adr,
   }
 
   // IDNA / RFC 3492 (Punycode) handling for HTTP(s)
-  if (!link_has_authority(adr) || strfield(adr, "https:")) {
-    char *const a = jump_identification(adr);
+  if (!link_has_authority(adrfil->adr) || strfield(adrfil->adr, "https:")) {
+    char *const a = jump_identification(adrfil->adr);
     // Non-ASCII characters (theorically forbidden, but browsers are lenient)
     if (!hts_isStringAscii(a, strlen(a))) {
       char *const idna = hts_convertStringUTF8ToIDNA(a, strlen(a));
@@ -392,7 +395,7 @@ int link_has_authority(const char *lien) {
 }
 
 int link_has_authorization(const char *lien) {
-  const char *adr = jump_protocol(lien);
+  const char *adr = jump_protocol_const(lien);
   const char *firstslash = strchr(adr, '/');
   const char *detect = strchr(adr, '@');
 
@@ -412,14 +415,15 @@ void long_to_83(int mode, char *n83, char *save) {
 
   while(*save) {
     char fn83[256], fnl[256];
-    int i = 0;
+    size_t i, j;
 
     fn83[0] = fnl[0] = '\0';
-    while((save[i]) && (save[i] != '/')) {
-      fnl[i] = save[i];
-      i++;
+    for(i = j = 0 ; save[i] && save[i] != '/' ; i++) {
+      if (j + 1 < sizeof(fnl)) {
+        fnl[j++] = save[i];
+      }
     }
-    fnl[i] = '\0';
+    fnl[j] = '\0';
     // conversion
     longfile_to_83(mode, fn83, fnl);
     strcatbuff(n83, fn83);
@@ -532,40 +536,39 @@ void longfile_to_83(int mode, char *n83, char *save) {
 // écrire backblue.gif
 /* Note: utf-8 */
 int verif_backblue(httrackp * opt, const char *base) {
-  int *done = &opt->state.verif_backblue_done;
   int ret = 0;
 
   //
   if (!base) {                  // init
-    *done = 0;
+    opt->state.verif_backblue_done = 0;
     return 0;
   }
-  if ((!*done)
-      || (fsize_utf8(fconcat(OPT_GET_BUFF(opt), base, "backblue.gif")) !=
+  if ((!opt->state.verif_backblue_done)
+      || (fsize_utf8(fconcat(OPT_GET_BUFF(opt), OPT_GET_BUFF_SIZE(opt), base, "backblue.gif")) !=
           HTS_DATA_BACK_GIF_LEN)) {
     FILE *fp =
       filecreate(&opt->state.strc,
-                 fconcat(OPT_GET_BUFF(opt), base, "backblue.gif"));
-    *done = 1;
+                 fconcat(OPT_GET_BUFF(opt), OPT_GET_BUFF_SIZE(opt), base, "backblue.gif"));
+    opt->state.verif_backblue_done = 1;
     if (fp) {
       if (fwrite(HTS_DATA_BACK_GIF, HTS_DATA_BACK_GIF_LEN, 1, fp) !=
           HTS_DATA_BACK_GIF_LEN)
         ret = 1;
       fclose(fp);
       usercommand(opt, 0, NULL,
-                  fconcat(OPT_GET_BUFF(opt), base, "backblue.gif"), "", "");
+                  fconcat(OPT_GET_BUFF(opt), OPT_GET_BUFF_SIZE(opt), base, "backblue.gif"), "", "");
     } else
       ret = 1;
     //
     fp =
       filecreate(&opt->state.strc,
-                 fconcat(OPT_GET_BUFF(opt), base, "fade.gif"));
+                 fconcat(OPT_GET_BUFF(opt), OPT_GET_BUFF_SIZE(opt), base, "fade.gif"));
     if (fp) {
       if (fwrite(HTS_DATA_FADE_GIF, HTS_DATA_FADE_GIF_LEN, 1, fp) !=
           HTS_DATA_FADE_GIF_LEN)
         ret = 1;
       fclose(fp);
-      usercommand(opt, 0, NULL, fconcat(OPT_GET_BUFF(opt), base, "fade.gif"),
+      usercommand(opt, 0, NULL, fconcat(OPT_GET_BUFF(opt), OPT_GET_BUFF_SIZE(opt), base, "fade.gif"),
                   "", "");
     } else
       ret = 1;
@@ -575,12 +578,12 @@ int verif_backblue(httrackp * opt, const char *base) {
 
 // flag
 int verif_external(httrackp * opt, int nb, int test) {
-  int *status = &opt->state.verif_external_status;
-
+  const int flag = 1 << nb;
+  int *const status = &opt->state.verif_external_status;
   if (!test)
-    status[nb] = 0;             // reset
-  else if (!status[nb]) {
-    status[nb] = 1;
+    *status &= ~flag;             // reset
+  else if ((*status & flag) == 0) {
+    *status |= flag;
     return 1;
   }
   return 0;
@@ -708,19 +711,18 @@ HTS_INLINE int rech_sampletag(const char *adr, const char *s) {
 }
 
 // teste si le tag contenu dans from est égal à "tag"
-HTS_INLINE int check_tag(char *from, const char *tag) {
-  char *a = from + 1;
+HTS_INLINE int check_tag(const char *from, const char *tag) {
+  const char *a = from + 1;
   int i = 0;
   char s[256];
 
   while(is_space(*a))
     a++;
-  while((isalnum((unsigned char) *a) || (*a == '/')) && (i < 250)) {
-    s[i++] = *a;
-    a++;
+  for( ; (isalnum((unsigned char) *a) || (*a == '/')) && i + 1 < sizeof(s) ; i++, a++) {
+    s[i] = *a;
   }
-  s[i++] = '\0';
-  return (strfield2(s, tag));   // comparer
+  s[i] = '\0';
+  return strfield2(s, tag);   // comparer
 }
 
 // teste si un fichier dépasse le quota
@@ -746,8 +748,8 @@ int istoobig(httrackp * opt, LLint size, LLint maxhtml, LLint maxnhtml,
 
 static int sortTopIndexFnc(const void *a_, const void *b_) {
   int cmp;
-  topindex_chain **a = (topindex_chain **) a_;
-  topindex_chain **b = (topindex_chain **) b_;
+  const topindex_chain *const*const a = (const topindex_chain *const*) a_;
+  const topindex_chain *const*const b = (const topindex_chain *const*) b_;
 
   /* Category first, then name */
   if ((cmp = (*a)->level - (*b)->level) == 0) {
@@ -758,7 +760,117 @@ static int sortTopIndexFnc(const void *a_, const void *b_) {
   return cmp;
 }
 
-HTSEXT_API char *hts_getcategory(const char *filename);
+typedef struct hts_template_format_buf {
+  FILE *fp;
+  char *buffer;
+  size_t size;
+  size_t offset;
+} hts_template_format_buf;
+
+// note: upstream arg list MUST be NULL-terminated for safety
+// returns a negative value upon error
+static int hts_template_formatv(hts_template_format_buf *buf, 
+                                const char *format, va_list args) {
+#undef FPUTC
+#undef FPUTS
+#define FPUTC(C) do { \
+  if (buf->fp != NULL) { \
+    assertf(buf->buffer == NULL); \
+    if (fputc(C, buf->fp) < 0) { \
+      return -1; \
+    } \
+  } else { \
+    assertf(buf->buffer != NULL); \
+    if (buf->offset + 1 < buf->size) { \
+      buf->buffer[buf->offset++] = (C); \
+    } else { \
+      return -1; \
+    } \
+  } \
+} while(0)
+#define FPUTS(S) do { \
+  size_t i; \
+  const char *const str_ = (S); \
+  assertf(str_ != NULL); \
+  for(i = 0 ; str_[i] != '\0' ; i++) { \
+    FPUTC(str_[i]); \
+  } \
+} while(0)
+
+  if (buf != NULL && format != NULL) {
+    const char *arg_expanded[32];
+    size_t i, nbArgs, posArgs;
+    /* Expand internal code args. */
+    const char *str;
+    for(nbArgs = 0 ; ( str = va_arg(args, const char*) ) != NULL ; nbArgs++) {
+      assertf(nbArgs < sizeof(arg_expanded)/sizeof(arg_expanded[0]));
+      arg_expanded[nbArgs] = str;
+    }
+    /* Expand user-injected format string. */
+    for(posArgs = 0, i = 0 ; format[i] != '\0' ; i++) {
+      const unsigned char c = format[i];
+      if (c == '%') {
+        const unsigned char cFormat = format[++i];
+        switch(cFormat) {
+        case '%':
+          FPUTC('%');
+          break;
+        case 's':
+          if (posArgs < nbArgs) {
+            assertf(arg_expanded[posArgs] != NULL);
+            FPUTS(arg_expanded[posArgs]);
+            posArgs++;
+          } else {
+            FPUTS("???");  /* error (args overflow) */
+          }
+          break;
+        default:  /* ignored */
+          FPUTC('%');
+          FPUTC(cFormat);
+          break;
+        }
+      } else {
+        FPUTC(c);
+      }
+    }
+    /* Terminating NULL. */
+    if (buf->buffer != NULL) {
+      buf->buffer[buf->offset] = 0;
+    }
+    return 1;
+  } else {
+    return -1;
+  }
+#undef FPUTC
+#undef FPUTS
+}
+
+// note: upstream arg list MUST be NULL-terminated for safety
+// returns a negative value upon error
+int hts_template_format(FILE *const out, const char *format, ...) {
+  int success;
+  hts_template_format_buf buf = { NULL, NULL, 0, 0 };
+  va_list args;
+  buf.fp = out;
+  va_start(args, format);
+  success = hts_template_formatv(&buf, format, args);
+  va_end(args);
+  return success;
+}
+
+// note: upstream arg list MUST be NULL-terminated for safety
+// returns a negative value upon error
+int hts_template_format_str(char *buffer, size_t size, const char *format, ...) {
+  int success;
+  hts_template_format_buf buf = { NULL, NULL, 0, 0 };
+  va_list args;
+  buf.buffer = buffer;
+  buf.size = size;
+  va_start(args, format);
+  success = hts_template_formatv(&buf, format, args);
+  va_end(args);
+  return success;
+}
 
 /* Note: NOT utf-8 */
 HTSEXT_API int hts_buildtopindex(httrackp * opt, const char *path,
@@ -772,16 +884,16 @@ HTSEXT_API int hts_buildtopindex(httrackp * opt, const char *path,
 
   // et templates html
   toptemplate_header =
-    readfile_or(fconcat(catbuff, binpath, "templates/topindex-header.html"),
+    readfile_or(fconcat(catbuff, sizeof(catbuff), binpath, "templates/topindex-header.html"),
                 HTS_INDEX_HEADER);
   toptemplate_body =
-    readfile_or(fconcat(catbuff, binpath, "templates/topindex-body.html"),
+    readfile_or(fconcat(catbuff, sizeof(catbuff), binpath, "templates/topindex-body.html"),
                 HTS_INDEX_BODY);
   toptemplate_bodycat =
-    readfile_or(fconcat(catbuff, binpath, "templates/topindex-bodycat.html"),
+    readfile_or(fconcat(catbuff, sizeof(catbuff), binpath, "templates/topindex-bodycat.html"),
                 HTS_INDEX_BODYCAT);
   toptemplate_footer =
-    readfile_or(fconcat(catbuff, binpath, "templates/topindex-footer.html"),
+    readfile_or(fconcat(catbuff, sizeof(catbuff), binpath, "templates/topindex-footer.html"),
                 HTS_INDEX_FOOTER);
 
   if (toptemplate_header && toptemplate_body && toptemplate_footer
@@ -793,15 +905,15 @@ HTSEXT_API int hts_buildtopindex(httrackp * opt, const char *path,
         rpath[strlen(rpath) - 1] = '\0';
     }
 
-    fpo = fopen(fconcat(catbuff, rpath, "/index.html"), "wb");
+    fpo = fopen(fconcat(catbuff, sizeof(catbuff), rpath, "/index.html"), "wb");
     if (fpo) {
       find_handle h;
 
-      verif_backblue(opt, concat(catbuff, rpath, "/")); // générer gif
+      verif_backblue(opt, concat(catbuff, sizeof(catbuff), rpath, "/")); // générer gif
       // Header
-      fprintf(fpo, toptemplate_header,
+      hts_template_format(fpo, toptemplate_header,
               "<!-- Mirror and index made by HTTrack Website Copier/"
-              HTTRACK_VERSION " " HTTRACK_AFF_AUTHORS " -->");
+              HTTRACK_VERSION " " HTTRACK_AFF_AUTHORS " -->", /* EOF */ NULL);
 
       /* Find valid project names */
       h = hts_findfirst(rpath);
@@ -870,7 +982,7 @@ HTSEXT_API int hts_buildtopindex(httrackp * opt, const char *path,
           assertf(sortedElts != NULL);
           if (sortedElts != NULL) {
             int i;
-            char *category = "";
+            const char *category = "";
 
             /* Build array */
             struct topindex_chain *chain = startchain;
@@ -887,15 +999,14 @@ HTSEXT_API int hts_buildtopindex(httrackp * opt, const char *path,
             for(i = 0; i < chainSize; i++) {
               char BIGSTK hname[HTS_URLMAXSIZE * 2];
 
-              strcpybuff(hname, sortedElts[i]->name);
-              escape_uri_utf(hname);
+              escape_uri_utf(sortedElts[i]->name, hname, sizeof(hname));
 
               /* Changed category */
               if (strcmp(category, sortedElts[i]->category) != 0) {
                 category = sortedElts[i]->category;
-                fprintf(fpo, toptemplate_bodycat, category);
+                hts_template_format(fpo, toptemplate_bodycat, category, /* EOF */ NULL);
               }
-              fprintf(fpo, toptemplate_body, hname, sortedElts[i]->name);
+              hts_template_format(fpo, toptemplate_body, hname, sortedElts[i]->name, /* EOF */ NULL);
             }
 
             /* Wipe elements */
@@ -913,9 +1024,9 @@ HTSEXT_API int hts_buildtopindex(httrackp * opt, const char *path,
 
       }
       // Footer
-      fprintf(fpo, toptemplate_footer,
+      hts_template_format(fpo, toptemplate_footer,
               "<!-- Mirror and index made by HTTrack Website Copier/"
-              HTTRACK_VERSION " " HTTRACK_AFF_AUTHORS " -->");
+              HTTRACK_VERSION " " HTTRACK_AFF_AUTHORS " -->", /* EOF */ NULL);
 
       fclose(fpo);
 
@@ -968,7 +1079,7 @@ HTSEXT_API char *hts_getcategories(char *path, int type) {
   String profiles = STRING_EMPTY;
   char *rpath = path;
   find_handle h;
-  inthash hashCateg = NULL;
+  coucal hashCateg = NULL;
 
   if (rpath[0]) {
     if (rpath[strlen(rpath) - 1] == '/') {
@@ -980,7 +1091,8 @@ HTSEXT_API char *hts_getcategories(char *path, int type) {
     String iname = STRING_EMPTY;
 
     if (type == 1) {
-      hashCateg = inthash_new(0);
+      hashCateg = coucal_new(0);
+      coucal_set_name(hashCateg, "hashCateg");
       StringCat(categ, "Test category 1");
       StringCat(categ, "\r\nTest category 2");
     }
@@ -1005,8 +1117,8 @@ HTSEXT_API char *hts_getcategories(char *path, int type) {
                 if (n > 0) {
                   if (strfield(line2, "category=")) {
                     if (*(line2 + 9)) {
-                      if (!inthash_read(hashCateg, line2 + 9, NULL)) {
-                        inthash_write(hashCateg, line2 + 9, 0);
+                      if (!coucal_read(hashCateg, line2 + 9, NULL)) {
+                        coucal_write(hashCateg, line2 + 9, 0);
                         if (StringLength(categ) > 0) {
                           StringCat(categ, "\r\n");
                         }
@@ -1034,7 +1146,7 @@ HTSEXT_API char *hts_getcategories(char *path, int type) {
     StringFree(iname);
   }
   if (hashCateg) {
-    inthash_delete(&hashCateg);
+    coucal_delete(&hashCateg);
     hashCateg = NULL;
   }
   if (type == 1)
@@ -1111,7 +1223,7 @@ HTSEXT_API int hts_findnext(find_handle find) {
     if ((find->dirp = readdir(find->hdir)))
       if (find->dirp->d_name)
         if (!STAT
-            (concat(catbuff, find->path, find->dirp->d_name), &find->filestat))
+            (concat(catbuff, sizeof(catbuff), find->path, find->dirp->d_name), &find->filestat))
           return 1;
 #endif
   }
