@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -42,10 +42,6 @@ Please visit our Website: http://www.httrack.com
 #include <ctype.h>
 /* END specific definitions */
 
-// pour alléger la syntaxe, des raccourcis sont créés
-#define urladr   (liens[ptr]->adr)
-#define urlfil   (liens[ptr]->fil)
-
 // libérer filters[0] pour insérer un élément dans filters[0]
 #define HT_INSERT_FILTERS0 do {\
   int i;\
@@ -60,8 +56,8 @@ Please visit our Website: http://www.httrack.com
 } while(0)
 
 typedef struct htspair_t {
-  char *tag;
-  char *attr;
+  const char *tag;
+  const char *attr;
 } htspair_t;
 
 /* "embedded" */
@@ -79,9 +75,9 @@ htspair_t hts_detect_embed[] = {
 };
 
 /* Internal */
-static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
-                           lien_url ** liens, char *adr, char *fil, char *tag,
-                           char *attribute, int *set_prio_to,
+static int hts_acceptlink_(httrackp * opt, int ptr, const char *adr,
+                           const char *fil, const char *tag,
+                           const char *attribute, int *set_prio_to,
                            int *just_test_it);
 
 /*
@@ -104,10 +100,11 @@ retour:
 -1 pas d'avis
 */
 
-int hts_acceptlink(httrackp * opt, int ptr, int lien_tot, lien_url ** liens,
-                   char *adr, char *fil, char *tag, char *attribute,
+int hts_acceptlink(httrackp * opt, int ptr,
+                   const char *adr, const char *fil,
+                   const char *tag, const char *attribute,
                    int *set_prio_to, int *just_test_it) {
-  int forbidden_url = hts_acceptlink_(opt, ptr, lien_tot, liens,
+  int forbidden_url = hts_acceptlink_(opt, ptr,
                                       adr, fil, tag, attribute, set_prio_to,
                                       just_test_it);
   int prev_prio = set_prio_to ? *set_prio_to : 0;
@@ -133,9 +130,9 @@ static int cmp_token(const char *tag, const char *cmp) {
           && !isalnum((unsigned char) tag[p]));
 }
 
-static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
-                           lien_url ** liens, char *adr, char *fil, char *tag,
-                           char *attribute, int *set_prio_to,
+static int hts_acceptlink_(httrackp * opt, int ptr,
+                           const char *adr, const char *fil, const char *tag,
+                           const char *attribute, int *set_prio_to,
                            int *just_test_it) {
   int forbidden_url = -1;
   int meme_adresse;
@@ -187,8 +184,8 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
 
   /* Niveau 1: ne pas parser suivant! */
   if (ptr > 0) {
-    if ((liens[ptr]->depth <= 0)
-        || (liens[ptr]->depth <= 1 && !embedded_triggered)) {
+    if ((heap(ptr)->depth <= 0)
+        || (heap(ptr)->depth <= 1 && !embedded_triggered)) {
       forbidden_url = 1;        // interdire récupération du lien
       hts_log_print(opt, LOG_DEBUG,
                     "file from too far level ignored at %s : %s", adr, fil);
@@ -203,11 +200,11 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
 
   // ------------------------------------------------------
   // doit-on traiter ce lien?.. vérifier droits de déplacement
-  meme_adresse = strfield2(adr, urladr);
+  meme_adresse = strfield2(adr, urladr());
   if (meme_adresse)
-    hts_log_print(opt, LOG_DEBUG, "Compare addresses: %s=%s", adr, urladr);
+    hts_log_print(opt, LOG_DEBUG, "Compare addresses: %s=%s", adr, urladr());
   else
-    hts_log_print(opt, LOG_DEBUG, "Compare addresses: %s!=%s", adr, urladr);
+    hts_log_print(opt, LOG_DEBUG, "Compare addresses: %s!=%s", adr, urladr());
   if (meme_adresse) {           // même adresse 
     {                           // tester interdiction de descendre
       // MODIFIE : en cas de remontée puis de redescente, il se pouvait qu'on ne puisse pas atteindre certains fichiers
@@ -220,15 +217,15 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       // note (up/down): on calcule à partir du lien primaire, ET du lien précédent.
       // ex: si on descend 2 fois on peut remonter 1 fois
 
-      if (lienrelatif(tempo, fil, liens[liens[ptr]->premier]->fil) == 0) {
-        if (lienrelatif(tempo2, fil, liens[ptr]->fil) == 0) {
+      if (lienrelatif(tempo, fil, heap(heap(ptr)->premier)->fil) == 0) {
+        if (lienrelatif(tempo2, fil, heap(ptr)->fil) == 0) {
           hts_log_print(opt, LOG_DEBUG,
                         "build relative links to test: %s %s (with %s and %s)",
-                        tempo, tempo2, liens[liens[ptr]->premier]->fil,
-                        liens[ptr]->fil);
+                        tempo, tempo2, heap(heap(ptr)->premier)->fil,
+                        heap(ptr)->fil);
 
           // si vient de primary, ne pas tester lienrelatif avec (car host "différent")
-          /*if (liens[liens[ptr]->premier] == 0) {   // vient de primary
+          /*if (heap(heap(ptr)->premier) == 0) {   // vient de primary
              }
            */
 
@@ -242,7 +239,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
               || (tempo2[0] != '\0' && tempo2[1] != '\0'
                   && strchr(tempo2 + 1, '/') == 0)
             ) {
-            if (!liens[ptr]->link_import) {     // ne résulte pas d'un 'moved'
+            if (!heap(ptr)->link_import) {     // ne résulte pas d'un 'moved'
               forbidden_url = 0;
               hts_log_print(opt, LOG_DEBUG, "same level link authorized: %s%s",
                             adr, fil);
@@ -267,7 +264,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
                 hts_log_print(opt, LOG_DEBUG, "lower link canceled: %s%s", adr,
                               fil);
               } else {          // autorisé à priori - NEW
-                if (!liens[ptr]->link_import) { // ne résulte pas d'un 'moved'
+                if (!heap(ptr)->link_import) { // ne résulte pas d'un 'moved'
                   forbidden_url = 0;
                   hts_log_print(opt, LOG_DEBUG, "lower link authorized: %s%s",
                                 adr, fil);
@@ -275,7 +272,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
               }
             } else if ((test1) || (test2)) {    // on peut descendre pour accéder au lien
               if ((opt->seeker & 1) != 0) {     // on peut descendre - NEW
-                if (!liens[ptr]->link_import) { // ne résulte pas d'un 'moved'
+                if (!heap(ptr)->link_import) { // ne résulte pas d'un 'moved'
                   forbidden_url = 0;
                   hts_log_print(opt, LOG_DEBUG, "lower link authorized: %s%s",
                                 adr, fil);
@@ -291,7 +288,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
               hts_log_print(opt, LOG_DEBUG, "upper link canceled: %s%s", adr,
                             fil);
             } else {            // autorisé à monter - NEW
-              if (!liens[ptr]->link_import) {   // ne résulte pas d'un 'moved'
+              if (!heap(ptr)->link_import) {   // ne résulte pas d'un 'moved'
                 forbidden_url = 0;
                 hts_log_print(opt, LOG_DEBUG, "upper link authorized: %s%s",
                               adr, fil);
@@ -299,7 +296,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
             }
           } else if ((!strncmp(tempo, "../", 3)) || (!strncmp(tempo2, "../", 3))) {     // Possible en montant
             if ((opt->seeker & 2) != 0) {       // autorisé à monter - NEW
-              if (!liens[ptr]->link_import) {   // ne résulte pas d'un 'moved'
+              if (!heap(ptr)->link_import) {   // ne résulte pas d'un 'moved'
                 forbidden_url = 0;
                 hts_log_print(opt, LOG_DEBUG, "upper link authorized: %s%s",
                               adr, fil);
@@ -310,11 +307,11 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
         } else {
           hts_log_print(opt, LOG_ERROR,
                         "Error building relative link %s and %s", fil,
-                        liens[ptr]->fil);
+                        heap(ptr)->fil);
         }
       } else {
         hts_log_print(opt, LOG_ERROR, "Error building relative link %s and %s",
-                      fil, liens[liens[ptr]->premier]->fil);
+                      fil, heap(heap(ptr)->premier)->fil);
       }
 
     }                           // tester interdiction de descendre?
@@ -323,16 +320,16 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       char BIGSTK tempo[HTS_URLMAXSIZE * 2];
       char BIGSTK tempo2[HTS_URLMAXSIZE * 2];
 
-      if (lienrelatif(tempo, fil, liens[liens[ptr]->premier]->fil) == 0) {
-        if (lienrelatif(tempo2, fil, liens[ptr]->fil) == 0) {
+      if (lienrelatif(tempo, fil, heap(heap(ptr)->premier)->fil) == 0) {
+        if (lienrelatif(tempo2, fil, heap(ptr)->fil) == 0) {
         } else {
           hts_log_print(opt, LOG_ERROR,
                         "Error building relative link %s and %s", fil,
-                        liens[ptr]->fil);
+                        heap(ptr)->fil);
         }
       } else {
         hts_log_print(opt, LOG_ERROR, "Error building relative link %s and %s",
-                      fil, liens[liens[ptr]->premier]->fil);
+                      fil, heap(heap(ptr)->premier)->fil);
 
       }
     }                           // fin tester interdiction de monter
@@ -348,24 +345,24 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       break;                    // interdicton de sortir au dela de l'adresse
     case 1:{                   // sortie sur le même dom.xxx
         size_t i = strlen(adr) - 1;
-        size_t j = strlen(urladr) - 1;
+        size_t j = strlen(urladr()) - 1;
 
         if ((i > 0) && (j > 0)) {
           while((i > 0) && (adr[i] != '.'))
             i--;
-          while((j > 0) && (urladr[j] != '.'))
+          while((j > 0) && (urladr()[j] != '.'))
             j--;
           if ((i > 0) && (j > 0)) {
             i--;
             j--;
             while((i > 0) && (adr[i] != '.'))
               i--;
-            while((j > 0) && (urladr[j] != '.'))
+            while((j > 0) && (urladr()[j] != '.'))
               j--;
           }
         }
         if ((i > 0) && (j > 0)) {
-          if (!strfield2(adr + i, urladr + j)) {        // !=
+          if (!strfield2(adr + i, urladr() + j)) {        // !=
             if (!opt->wizard) { // mode non wizard
               //printf("refused: %s\n",adr);
               forbidden_url = 1;        // pas même domaine  
@@ -387,14 +384,14 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       break;
     case 2:{                   // sortie sur le même .xxx
         size_t i = strlen(adr) - 1;
-        size_t j = strlen(urladr) - 1;
+        size_t j = strlen(urladr()) - 1;
 
         while((i > 0) && (adr[i] != '.'))
           i--;
-        while((j > 0) && (urladr[j] != '.'))
+        while((j > 0) && (urladr()[j] != '.'))
           j--;
         if ((i > 0) && (j > 0)) {
-          if (!strfield2(adr + i, urladr + j)) {        // !-
+          if (!strfield2(adr + i, urladr() + j)) {        // !-
             if (!opt->wizard) { // mode non wizard
               //printf("refused: %s\n",adr);
               forbidden_url = 1;        // pas même .xx  
@@ -462,7 +459,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       question = 0;             // pas de question, résolu
 
     // former URL complète du lien actuel
-    strcpybuff(l, jump_identification(adr));
+    strcpybuff(l, jump_identification_const(adr));
     if (*fil != '/')
       strcatbuff(l, "/");
     strcatbuff(l, fil);
@@ -493,7 +490,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
           // there is no way yet to tag "external" links, and therefore links that are
           // "weak" (authorized depth < external depth) are just not considered for external
           // hack
-          if (liens[ptr]->depth > opt->extdepth) {
+          if (heap(ptr)->depth > opt->extdepth) {
             // *set_prio_to = opt->extdepth + 1;
             *set_prio_to = 1 + (opt->extdepth);
             may_set_prio_to = 0;        // clear may-set flag
@@ -502,11 +499,11 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
             if (question) {
               hts_log_print(opt, LOG_DEBUG,
                             "(wizard) ambiguous link accepted (external depth): link %s at %s%s",
-                            l, urladr, urlfil);
+                            l, urladr(), urlfil());
             } else {
               hts_log_print(opt, LOG_DEBUG,
                             "(wizard) forced to accept link (external depth): link %s at %s%s",
-                            l, urladr, urlfil);
+                            l, urladr(), urlfil());
             }
 
           }
@@ -515,7 +512,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       // filters
       {
         int jok;
-        char *mdepth = "";
+        const char *mdepth = "";
 
         // filters, 0=sait pas 1=ok -1=interdit
         {
@@ -550,14 +547,14 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
           may_set_prio_to = 0;  // clear may-set flag
           hts_log_print(opt, LOG_DEBUG,
                         "(wizard) explicit authorized (%s) link: link %s at %s%s",
-                        mdepth, l, urladr, urlfil);
+                        mdepth, l, urladr(), urlfil());
         } else if (jok == -1) { // forbidden
           filters_answer = 1;   // décision prise par les filtres
           question = 0;         // ne pas poser de question:
           forbidden_url = 1;    // URL interdite
           hts_log_print(opt, LOG_DEBUG,
                         "(wizard) explicit forbidden (%s) link: link %s at %s%s",
-                        mdepth, l, urladr, urlfil);
+                        mdepth, l, urladr(), urlfil());
         }                       // sinon on touche à rien
       }
     }
@@ -565,14 +562,14 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
     // vérifier mode mirror links
     if (question) {
       if (opt->mirror_first_page) {     // mode mirror links
-        if (liens[ptr]->precedent == 0) {       // parent=primary!
+        if (heap(ptr)->precedent == 0) {       // parent=primary!
           forbidden_url = 0;    // autorisé
           may_set_prio_to = 0;  // clear may-set flag
           question = 1;         // résolution auto
           force_mirror = 5;     // mirror (5)
           hts_log_print(opt, LOG_DEBUG,
                         "(wizard) explicit mirror link: link %s at %s%s", l,
-                        urladr, urlfil);
+                        urladr(), urlfil());
         }
       }
     }
@@ -584,7 +581,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
         forbidden_url = 1;
         hts_log_print(opt, LOG_DEBUG,
                       "(wizard) ambiguous forbidden link: link %s at %s%s", l,
-                      urladr, urlfil);
+                      urladr(), urlfil());
       }
     }
     // vérifier robots.txt
@@ -619,14 +616,14 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       if (!forbidden_url) {
         hts_log_print(opt, LOG_DEBUG,
                       "(wizard) shared foreign domain link: link %s at %s%s", l,
-                      urladr, urlfil);
+                      urladr(), urlfil());
       } else {
         hts_log_print(opt, LOG_DEBUG,
                       "(wizard) cancelled foreign domain link: link %s at %s%s",
-                      l, urladr, urlfil);
+                      l, urladr(), urlfil());
       }
 #if BDEBUG==3
-      printf("at %s in %s, wizard says: url %s ", urladr, urlfil, l);
+      printf("at %s in %s, wizard says: url %s ", urladr(), urlfil(), l);
       if (forbidden_url)
         printf("cancelled");
       else
@@ -714,7 +711,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
         forbidden_url = 1;
         HT_INSERT_FILTERS0;     // insérer en 0
         strcpybuff(_FILTERS[0], "-");
-        strcatbuff(_FILTERS[0], jump_identification(adr));
+        strcatbuff(_FILTERS[0], jump_identification_const(adr));
         if (*fil != '/')
           strcatbuff(_FILTERS[0], "/");
         strcatbuff(_FILTERS[0], fil);
@@ -730,7 +727,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
           if (fil[i] == '/') {
             HT_INSERT_FILTERS0; // insérer en 0
             strcpybuff(_FILTERS[0], "-");
-            strcatbuff(_FILTERS[0], jump_identification(adr));
+            strcatbuff(_FILTERS[0], jump_identification_const(adr));
             if (*fil != '/')
               strcatbuff(_FILTERS[0], "/");
             strncatbuff(_FILTERS[0], fil, i);
@@ -747,7 +744,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
         forbidden_url = 1;
         HT_INSERT_FILTERS0;     // insérer en 0                                
         strcpybuff(_FILTERS[0], "-");
-        strcatbuff(_FILTERS[0], jump_identification(adr));
+        strcatbuff(_FILTERS[0], jump_identification_const(adr));
         strcatbuff(_FILTERS[0], "*");
         break;
 
@@ -789,7 +786,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
           if (fil[i] == '/') {
             HT_INSERT_FILTERS0; // insérer en 0                                
             strcpybuff(_FILTERS[0], "+");
-            strcatbuff(_FILTERS[0], jump_identification(adr));
+            strcatbuff(_FILTERS[0], jump_identification_const(adr));
             if (*fil != '/')
               strcatbuff(_FILTERS[0], "/");
             strncatbuff(_FILTERS[0], fil, i + 1);
@@ -798,7 +795,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
         } else {                // autoriser domaine alors!!
           HT_INSERT_FILTERS0;   // insérer en 0                                strcpybuff(filters[filptr],"+");
           strcpybuff(_FILTERS[0], "+");
-          strcatbuff(_FILTERS[0], jump_identification(adr));
+          strcatbuff(_FILTERS[0], jump_identification_const(adr));
           strcatbuff(_FILTERS[0], "*");
         }
         break;
@@ -806,7 +803,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
       case 6:                  // same domain
         HT_INSERT_FILTERS0;     // insérer en 0                                strcpybuff(filters[filptr],"+");
         strcpybuff(_FILTERS[0], "+");
-        strcatbuff(_FILTERS[0], jump_identification(adr));
+        strcatbuff(_FILTERS[0], jump_identification_const(adr));
         strcatbuff(_FILTERS[0], "*");
         break;
         //
@@ -819,7 +816,7 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
           if (fil[i] == '/') {
             HT_INSERT_FILTERS0; // insérer en 0                                
             strcpybuff(_FILTERS[0], "+");
-            strcatbuff(_FILTERS[0], jump_identification(adr));
+            strcatbuff(_FILTERS[0], jump_identification_const(adr));
             if (*fil != '/')
               strcatbuff(_FILTERS[0], "/");
             strncatbuff(_FILTERS[0], fil, i + 1);
@@ -863,13 +860,13 @@ static int hts_acceptlink_(httrackp * opt, int ptr, int lien_tot,
 #undef _ROBOTS
 }
 
-int hts_acceptmime(httrackp * opt, int ptr, int lien_tot, lien_url ** liens,
-                   char *adr, char *fil, char *mime) {
+int hts_acceptmime(httrackp * opt, int ptr,
+                   const char *adr, const char *fil, const char *mime) {
 #define _FILTERS     (*opt->filters.filters)
 #define _FILTERS_PTR (opt->filters.filptr)
 #define _ROBOTS      ((robots_wizard*)opt->robotsptr)
   int forbidden_url = -1;
-  char *mdepth = "";
+  const char *mdepth = "";
   int jokDepth = 0;
   int jok = 0;
 
@@ -906,7 +903,7 @@ int hts_acceptmime(httrackp * opt, int ptr, int lien_tot, lien_url ** liens,
 }
 
 // tester taille
-int hts_testlinksize(httrackp * opt, char *adr, char *fil, LLint size) {
+int hts_testlinksize(httrackp * opt, const char *adr, const char *fil, LLint size) {
   int jok = 0;
 
   if (size >= 0) {
@@ -918,7 +915,7 @@ int hts_testlinksize(httrackp * opt, char *adr, char *fil, LLint size) {
       int size_flag = 0;
 
       // former URL complète du lien actuel
-      strcpybuff(l, jump_identification(adr));
+      strcpybuff(l, jump_identification_const(adr));
       if (*fil != '/')
         strcatbuff(l, "/");
       strcatbuff(l, fil);
@@ -984,7 +981,4 @@ int hts_testlinksize(httrackp * opt, char *adr, char *fil, LLint size) {
   return jok;
 }
 
-#undef urladr
-#undef urlfil
-
 #undef HT_INSERT_FILTERS0
