@@ -69,8 +69,9 @@ static int linput(FILE * fp, char *s, int max);
 #include <unistd.h>
 #endif
 #include <ctype.h>
-#ifdef __linux
+#if (defined(__linux) && defined(HAVE_EXECINFO_H))
 #include <execinfo.h>
+#define USES_BACKTRACE
 #endif
 /* END specific definitions */
 
@@ -878,7 +879,7 @@ static void sig_doback(int blind) {     // mettre en backing
 #define FD_ERR 2
 
 static void print_backtrace(void) {
-#ifdef __linux
+#ifdef USES_BACKTRACE
   void *stack[256];
   const int size = backtrace(stack, sizeof(stack)/sizeof(stack[0]));
   if (size != 0) {
